#!/usr/bin/python2

import json
import time
import pycurl
import socket
from dmliteshell import executor
import argparse
import sys

# The top level object
class storageService:

  def __init__(self, config):
    self.entry={}
    self.entry["name"]=config.host
    self.entry["implementation"]="DPM"
    self.entry["qualitylevel"]="production"
    self.entry["storagecapacity"]="Not implemented"
    self.entry["storageshares"]=[]
    self.entry["storageendpoints"]=[]
    self.entry["datastores"]=[]
    self.entry["latestupdate"]=int(time.time())

  def addshares(self):
    e=executor.DomeExecutor(config.cert, config.key, "/etc/grid-security/certificates/", "", "")
    gqt, err=e.getquotatoken("https://"+config.host+":"+str(config.domeport)+"/domehead/command","/", 0, 1)
    if err==True:
      print "Error contacting Dome"
      sys.exit(1)
    try:
      jgqt=json.loads(gqt)
    except Exception as X:
      print "Problem loading Dome output"
      print X
      sys.exit(1)
    for space in jgqt.keys():
      if isinstance(jgqt[space]["groups"], dict):
        groups = jgqt[space]["groups"].values()
      if isinstance(jgqt[space]["groups"], list):
        groups = jgqt[space]["groups"]
      groups.remove("root")
      self.entry["storageshares"].append(storageShare(jgqt[space]["quotatkname"],jgqt[space]["quotatktotspace"], jgqt[space]["pathusedspace"], jgqt[space]["path"], groups))

  def addendpoints(self):
    self.entry["storageendpoints"].append(storageEndpoint("gsiftp", "gsiftp://"+config.host+"/", "gsiftp"))
    self.entry["storageendpoints"].append(storageEndpoint("https", "https://"+config.host+"/", "https"))
    self.entry["storageendpoints"].append(storageEndpoint("xrootd", "root://"+config.host+"/", "xrootd"))
    self.entry["storageendpoints"].append(storageEndpoint("srm", "srm://"+config.host+":8446/srm/managerv2?SFN=/", "srm"))

  def adddataStores(self):
    self.entry["datastores"].append(dataStore("DPM data store"))

  def printjson(self):
    print json.dumps({"storageservice" : self.entry}, indent=4)

  def publish(self):
    c=pycurl.Curl()
    c.setopt(c.SSLCERT, config.cert)
    c.setopt(c.SSLKEY, config.key)
    c.setopt(c.SSL_VERIFYPEER, 0)
    c.setopt(c.SSL_VERIFYHOST, 2)
    c.setopt(c.FOLLOWLOCATION, 1)
    c.setopt(c.URL, 'https://'+config.host+config.path+"/storagesummary.json")
    # delete existing file, this will fail the first time...
    c.setopt(c.CUSTOMREQUEST, "DELETE")
    c.perform()
    # now put the new file
    c.setopt(c.CUSTOMREQUEST, "PUT")
    c.setopt(c.POSTFIELDS, json.dumps({"storageservice" : self.entry}, indent=4))
    try:
      c.perform()
    except Exception as e:
      print(e)
    c.close()

# Derive from dict for easy serialisation    
class storageEndpoint(dict):
  def __init__(self, name, endpointurl, interfacetype):
    self["name"]=name
    self["endpointurl"]=endpointurl
    self["interfacetype"]=interfacetype
    self["qualitylevel"]="production"
    self["assignedshares"]=["all"]

# Derive from dict for easy serialisation    
class storageShare(dict):
  def __init__(self, name, totalsize, usedsize, path, groups):
    self["name"]=name
    self["timestamp"]=int(time.time())
    self["totalsize"]=int(totalsize)
    self["usedsize"]=int(usedsize)
    self["path"]=[path]
    self["vos"]=groups
    self["assignedendpoints"]=["all"]
    self["servingstate"]="open"
    
# Derive from dict for easy serialisation    
class dataStore(dict):
  def __init__(self, name):
    self["name"]=name
    self["message"]="Not used"


# Quick config on the fly    
class config:
    pass

# Arguments
parser = argparse.ArgumentParser()
parser.add_argument("--path", help="Path to publish the summary file", default="/dpm/cern.ch/home/dteam")
parser.add_argument("--cert", help="Path to host certificate", default="/etc/grid-security/hostcert.pem")
parser.add_argument("--key", help="Path to host key", default="/etc/grid-security/hostkey.pem")
parser.add_argument("--host", help="FQDN", default=socket.getfqdn())
parser.add_argument("--print", help="Just print, don't publish", action='store_true', dest='prnt')
parser.add_argument("--port", help="Dome's port number", default=1094)

args = parser.parse_args()

config=config()
config.path=args.path
config.cert=args.cert
config.key=args.key
config.host=args.host
config.domeport=args.port

# Create object
dpm=storageService(config)
dpm.addshares()

# Which endpoints are supposed to be there?
dpm.addendpoints()
# Don't bother with this for now
#dpm.adddataStores()

# Print or publish
if args.prnt:
  dpm.printjson()
else:
  dpm.publish()
